/*
 *   This file is part of the MLV Library.
 *
 *   Copyright (C) 2010 Adrien Boussicault, Marc Zipstein
 *
 *
 *    This Library is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This Library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this Library.  If not, see <http://www.gnu.org/licenses/>.
 */

/** \~french 
 * \file MLV_animation.h
 *
 * \author Adrien Boussicault
 * \author Marc Zipstein
 *
 * Cette interface est inspiré du tutoriel : 
 * http://www.gnurou.org/writing/linuxmag/sdl/partie3
 *
 * \brief Ce fichier définit les prototypes des fonctions permettant de mettre 
 *        en oeuvre des animations.
 */

#ifndef __MLV_ANIMATION_H__
#define __MLV_ANIMATION_H__

#include "MLV_image.h"
#include "MLV_audio.h"

#ifdef __cplusplus
extern "C" {
#endif

////////////////////////////////////////////////
//    Animation data
///////////////////////////////////////////////

/** \~french 
 * \brief Définit le type Animation qui code une animation dans la librairie 
 *        MLV.
 */
typedef struct _MLV_Animation_sequence MLV_Animation_sequence;

/** \~french 
 * \brief Créé une séquence d'animation comportant un nombre d'image donné
 *
 * Une séquence d'animation est une succession d'image où à chaque image est 
 * associé un temps. Ce temps code le temps d'affichage de chaque image, 
 * lors du rendu de de l'animation.
 * Pour des raisons de commodité, nos séquence d'animations sont composés de 
 * plusieurs  couches. Elles sont donc des successions de couches contenant 
 * chacunes une image.
 *
 * Une sequence d'animation (à ne pas confondre avec une animation) est une 
 * pure structure de donnée. Elle peut donc pas s'animer, ni s'afficher à 
 * l'ecran.
 * Pour afficher une séquence d'animation, il faut utiliser une animation, 
 * qui s'aidera de la séquence d'animation pour s'afficher ou pour mettre en
 * mouvement les images de la séquence d'animation.
 *
 * Le temps d'une s'quence d'animation n'est pas codée en secondes. Il s'agit 
 * d'un entier. Lorsque vous utilisez une animation, l'animateur à un temps qui
 * lui est propre et qui est codé à l'aide d'un entier qui croit. Les duree que
 * vous ajouterez dans l'animation sont donc définit par rapport au temps de 
 * l'animation qui utilise la séquence d'animation.
 *
 * \param nb_frames Le nombre d'image que doit contenir l'animation
 * \param nb_layers Nombre de couches graphique de l'animation.
 * \param nb_channels Nombre de canaux sonore de l'animation.
 * \return Un pointeur vers une animation.
 */
MLV_Animation_sequence* MLV_create_animation_sequence(
	unsigned int nb_frames,	unsigned int nb_layers,	unsigned int nb_channels
);

/** \~french 
 * \brief Libère les données alloué pour l'animation
 */
void MLV_free_animation_sequence( MLV_Animation_sequence* animation );

/** \~french 
 * \brief Change un animation en remplacant une image et son temps d'affichage 
 *        associé. L'image concerné est repéré à l'aide de son index et de
 *        l'index de la couche où elle appartient.
 *
 * \param array_of_images Un tableau d'adresse contenant les addresses de images
 *                        correspondat aux différentes couches d'une animation.
 * \param array_of_sounds Un tableau d'adresse contenant les addresses de sons
 *                        correspondant aux différentes canaux d'une animation.
 * \param delay temps d'affichage de l'image
 * \param animation Animation à modifier
 * \param position position de l'image dans l'animation
 */
void MLV_change_frame_in_animation_sequence(
	MLV_Image** array_of_images, MLV_Sound** array_of_sounds, unsigned int delay,
	MLV_Animation_sequence* animation, unsigned int position
);

/** \~french 
 * \brief Change un animation en remplacant uniquement une image.
 *        L'image concerné est repéré à l'aide de son index et de
 *        de l'index de sa couche.
 *
 * \param image image a ajouter dans l'animation
 * \param animation Animation à modifier
 * \param position position de l'image dans l'animation
 * \param layer     Couche de l'image.
 */
void MLV_change_frame_image_in_animation_sequence(
	MLV_Image* image, MLV_Animation_sequence* animation, unsigned int position,
	unsigned int layer
);

/** \~french 
 * \brief Change un animation en remplacant uniquement un bruitage.
 *        Le son image concerné est repéré à l'aide de son index et de
 *        de l'index de son canaul.
 *
 * \param sound     son a ajouter dans l'animation
 * \param animation Animation à modifier
 * \param position  position de l'image dans l'animation
 * \param channel   Canal du son.
 */
void MLV_change_frame_sound_in_animation_sequence(
	MLV_Sound* sound, MLV_Animation_sequence* animation, unsigned int position,
	unsigned int channel
);

/** \~french 
 * \brief Change le temps d'affichage d'une image donnée d'une animation.
 *
 * \param delay temps d'affichage de l'image
 * \param animation Animation à modifier
 * \param position position de l'image dans l'animation
 */
void MLV_change_frame_delay_in_animation_sequence(
	unsigned int delay, MLV_Animation_sequence* animation, unsigned int position
);

/** \~french 
 * \brief Recupère l'image et le temps présents à une position et une couche 
 *        donnée d'une animation.
 *
 * \param animation Animation à modifier
 * \param layer Couche de l'image.
 * \param position position de l'image dans l'animation
 * \param image image a ajouter dans l'animation
 * \param delay temps d'affichage de l'image
 */
void MLV_get_frame_from_animation_sequence(
	MLV_Animation_sequence* animation, unsigned int position,
	unsigned int layer,
	MLV_Image** image, unsigned int* delay
);

/** \~french 
 * \brief Renvoie le temps d'affichage de l'image  présente à une position 
 *        donnée d'une animation donnée.
 *
 * \param animation Animation à modifier
 * \param position position de l'image dans l'animation
 *
 * \return le temps d'affichage de l'image
 */
int MLV_get_frame_delay_from_animation_sequence(
	MLV_Animation_sequence* animation, unsigned int position
);

/** \~french 
 * \brief Renvoie l'image présente à une position et une couche donnée d'une
 *        animation donnée.
 *
 * \param animation Animation à modifier
 * \param position position de l'image dans l'animation
 * \param layer Couche de l'image.
 *
 * \return l'image extraite.
 */
MLV_Image* MLV_get_frame_image_from_animation_sequence(
	MLV_Animation_sequence* animation, unsigned int position,
	unsigned int layer
);




////////////////////////////////////////////////
//    Animation engine
///////////////////////////////////////////////

/** \~french 
 * \brief Définit le type d'un animateur dans la librairie 
 *        MLV.
 *
 * Un animateur est un objet qui garde contient les informations suivante:
 *   - un pointeur vers une animation
 *   - la position de l'image courante de l'animation
 *   - une horloge interne ( dons le temps est différent du temps courant )
 *
 * Un animateur permet d'animer une animation.
 * Pour cela, l'animateur possède une horloge dont l'aiguille avance d'une unité
 * à chaque appel de la fonction update.
 * Les animations sont alors jouée en fonction de l'évolution du temps de chaque
 * animateur. Ainsi, deux animateurs différents peuvent avoir une courbe du
 * temps différente.
 *
 * La bonne façon d'utiliser des animateurs est de mettre a jour tous les
 * animateurs en même temps à un intervalle de temps régulier.
 * Pour cela vous pouvez utiliser la fonction framerate qui endort le programme 
 * de façon a assurer l'execution d'un tour de boucle à une fréquence donnée
 *
 * \code
 * MLV_Image* creature1 = MLV_load_image("creature1.png");
 * MLV_Image* creature2 = MLV_load_image("creature2.png");
 *
 * MLV_Animation_sequence* animation_seq;
 * animation_seq = MLV_create_animation_sequence( 2 );
 * MLV_change_frame_in_animation_sequence( creature1, 1, animation_seq, 0 );
 * MLV_change_frame_in_animation_sequence( creature2, 1, animation_seq, 0 );
 *
 * MLV_Animation* animation; 
 * animation = MLV_create_animation( animation );
 *
 * int frequence = 24; // nombre d'images par secondes
 * MLV_change_frame_rate( frequence );
 * while( 1 ){ 
 *     MLV_animation_update( animation );
 *     MLV_draw_animation(animation, 0, 0);
 *     MLV_frame_rate_delay();
 * }
 * \endcode
 *
 */
typedef struct _MLV_Animation MLV_Animation;


/** \~french 
 * \brief Cree un animateur et l'initialise avec une animation donnée en
 *        paramètre.
 *
 * \param animation L'animation que devra animer l'animateur.
 *
 * \return L'animateur créé.
 */
MLV_Animation* MLV_create_animation( MLV_Animation_sequence* animation );

/** \~french 
 * \brief Libère l'espace mémoire alloué pour un animateur donné.
 *
 * \param animation L'animateur à suprimer
 */
void MLV_free_animation( MLV_Animation* animation );

/** \~french 
 * \brief Demare la lecture de l'animation lu par l'animateur.
 *
 * \param animation L'animateur
 */
void MLV_play_animation( MLV_Animation* animation );

/** \~french 
 * \brief Demande à un animateur donné de jouer l'animation à l'envers.
 *
 * \param animation L'animateur
 */
void MLV_play_revert_animation( MLV_Animation* animation );

/** \~french 
 * \brief Met en pause la lecture d'un animateur donné.
 *
 * \param animation L'animateur
 */
void MLV_stop_animation( MLV_Animation* animation );

/** \~french 
 * \brief Demande à l'animateur de reprendre la lecture de l'animation depuis le
 *        début.
 *
 * \param animation L'animateur
 */
void MLV_rewind_animation( MLV_Animation* animation );

/** \~french 
 * \brief Force l'animateur à passer à l'image suivante.
 *
 * \param animation L'animateur
 */
void MLV_next_frame( MLV_Animation* animation );

/** \~french 
 * \brief Force l'animateur à revenir sur l'image précédente.
 *
 * \param animation L'animateur
 */
void MLV_previous_frame( MLV_Animation* animation );

/** \~french 
 * \brief Met a jour l'animateur.
 *
 * L'horloge interne de l'animateur vois son temps incrémenter de 1 unité
 * exactement.
 *
 * La position de lecture de l'animation est mis a jour en conséquence de la 
 * mis à jour de l'horloge interne.
 *
 * \param animation L'animateur à mettre à jour
 */
void MLV_update_animation( MLV_Animation* animation );

/** \~french 
 * \brief Change le volume sonore d'un bruitage.
 *
 * \param animation L'animateur dans lequel se trouve le son.
 * \param channel Le canal concerné.
 * \param volume le nouveau volume sonore.
 */
void MLV_change_sound_volume_on_animation(
	MLV_Animation* animation, unsigned int channel, float volume
);

/** \~french 
 * \brief Active le son du canal d'une animation donnée.
 *
 * \param animation L'animateur concerné.
 * \param channel Le canal concerné.
 */
void MLV_turn_on_sound_on_animation(
	MLV_Animation* animation, unsigned int channel
);

/** \~french 
 * \brief Désactive le son du canal d'une animation donnée.
 *
 * \param animation L'animateur concerné.
 * \param channel Le canal concerné.
 */
void MLV_turn_off_sound_on_animation(
	MLV_Animation* animation, unsigned int channel
);


////////////////////////////////////////////////
//    Drawing animation
///////////////////////////////////////////////

/** \~french 
 * \brief Dessine à une position donnée l'image actuellement lue par 
 *        l'animateur. Cette image est situè à une couche donnée en paramètre.
 *
 * \param animation L'animateur contenant l'animation à aficher
 * \param layer     Couche de l'image.
 * \param x         Coordonnée en X de la position du sommet Nord-Ouest de
 *                  l'image à afficher.
 * \param y         Coordonnée en Y de la position du sommet Nord-Ouest de
 *                  l'image à afficher.
 */
void MLV_draw_animation(
	MLV_Animation* animation, unsigned int layer,
	int x, int y
);

/** \~french 
 * \brief Dessine à l'écran à une position donnée, une portion de 
 *        l'image actuellement lue par l'animateur et extraite de la 
 *        couche spécifiée en paramètre.
 *
 * \param animation L'animateur contenant l'animation à aficher
 * \param layer     Couche de l'image.
 * \param image    image sur laquel il faut dessiner l'animation
 * \param source_x       Coordonnée en X de la position de la portion à recopier
 * \param source_y       Coordonnée en Y de la position de la portion à recopier
 * \param source_width   Largeur de la portion à recopier
 * \param source_height  Hauteur de la portion à recopier
 * \param x        Coordonnée en X de la position du sommet Nord-Ouest de
 *                 l'image à afficher.
 * \param y        Coordonnée en Y de la position du sommet Nord-Ouest de
 *                 l'image à afficher.
 */
void MLV_draw_partial_animation(
	MLV_Animation* animation, unsigned int layer,
	int source_x, int source_y, int source_width, int source_height,
	MLV_Image* image,
	int x, int y
);

/** \~french 
 * \brief Dessine sur une image donnée à une position donnée l'image 
 *        actuellement lue par l'animateur et située à une couche donnée.
 *
 * \param animation L'animateur contenant l'animation à aficher
 * \param layer     Couche de l'image.
 * \param image    image sur laquel il faut dessiner l'animation
 * \param x        Coordonnée en X de la position du sommet Nord-Ouest de
 *                 l'image à afficher.
 * \param y        Coordonnée en Y de la position du sommet Nord-Ouest de
 *                 l'image à afficher.
 */
void MLV_draw_animation_on_image(
	MLV_Animation* animation, unsigned int layer,
	MLV_Image* image,
	int x, int y
);

/** \~french 
 * \brief Dessine sur une image donnée à une position donnée une portion de 
 *        l'image actuellement lue par l'animateur et située à une couche
 *        donnée.
 *
 * \param animation L'animateur contenant l'animation à aficher
 * \param layer     Couche de l'image.
 * \param image    image sur laquel il faut dessiner l'animation
 * \param source_x       Coordonnée en X de la position de la portion à recopier
 * \param source_y       Coordonnée en Y de la position de la portion à recopier
 * \param source_width   Largeur de la portion à recopier
 * \param source_height  Hauteur de la portion à recopier
 * \param x        Coordonnée en X de la position du sommet Nord-Ouest de
 *                 l'image à afficher.
 * \param y        Coordonnée en Y de la position du sommet Nord-Ouest de
 *                 l'image à afficher.
 */
void MLV_draw_partial_animation_on_image(
	MLV_Animation* animation, unsigned int layer,
	int source_x, int source_y, int source_width, int source_height,
	MLV_Image* image,
	int x, int y
);




////////////////////////////////////////////////
//    Loading animations from files
///////////////////////////////////////////////

/** \~french 
 * \brief Définit le type Animation_bbook qui code un livre contenant de 
 *        nombreuses animations.
 */
typedef struct _MLV_Animation_book MLV_Animation_book;


/** \~french 
 * \brief Charge en memoire un livre contenant une multitude d'animations.
 *
 * \bug Lorsque le fichier xml contient des données corrompus, la fonction ne le
 *      detecte pas, continu de travailler avec les données corrompu et finie
 *      par faire une erreur de segmentation.
 *
 * \param xml_file Fichiers xml.
 * \param image_directory Le dossier ou se trouve toutes les images du livre 
 *        d'animation. Si NULL est passe en parametre, alors le programme 
 *        cherche dans le repertoire du fichier de configuration du livre 
 *        d'animation.
 * \param sound_directory Le dossier ou se trouve toutes les échantillons
 *        sonore du livre d'animation. Si NULL est passe en parametre, 
 *        alors le programme cherche dans le repertoire du fichier de 
 *        configuration du livre d'animation.
 * \return L'adresse de la sructure de données contenant toutes les animations.
 */
MLV_Animation_book* MLV_load_animation_book(
	const char* xml_file,
	const char* image_directory,
	const char* sound_directory
);

/** \~french 
 * \brief Libère la memoire contenue par le livre d'animation.
 *
 * \param animation_book Le livre d'animation.
 */
void MLV_free_animation_book(
	MLV_Animation_book* animation_book
);

/** \~french 
 * \brief Détermine le nombre d'animations contenus par le livre d'animation.
 *
 * \param animation_book Le livre d'animation.
 * \return le nombre d'animations contenus par le livre d'animation.
 */
int MLV_get_number_of_animation_sequences(
	MLV_Animation_book* animation_book
);

/** \~french 
 * \brief Renvoie une animation donnée contenue dans le livre d'animation.
 *
 * \param animation_book Le livre d'animation.
 * \param id L'identifiant
 * \return La séquence d'animation.
 */
MLV_Animation_sequence* MLV_get_animation_from_id(
	MLV_Animation_book* animation_book,
	int id
);

/** \~french 
 * \brief Renvoie une animation donnée contenue dans le livre d'animation.
 *
 * \param animation_book Le livre d'animation.
 * \param name Le nom de l'animation
 * \return La séquence d'animation.
 */
MLV_Animation_sequence* MLV_get_animation_from_name(
	MLV_Animation_book* animation_book,
	const char* name
);

/** \~french 
 * \brief Renvoie le nom d'un animation à partir de son identifiant dans le
 *        livre d'animation.
 *
 * \param animation_book Le livre d'animation.
 * \param id_animation L'identifiant de l'animation dans le livre.
 * \return Le nom de l'animation.
 */
const char* MLV_get_name_from_id_animation(
	MLV_Animation_book* animation_book, int id_animation
);

/** \~french 
 * \brief Ajoute un image dans la séquence d'animation.
 *        Cette fonction ne peux pas augmenter le nombre d'image d'une
 *        animation.
 *
 * \param array_of_images Liste d'image à ajouter. Il s'agit de toutes les 
 *                        images assiciée à toutes les couches correspondnant de
 *                        l'animation.
 * \param array_of_sounds Liste de sons à ajouter. Il s'agit de toutes les 
 *                        sons assiciées à tous les cannaux correspondnant de
 *                        l'animation.
 * \param delay le temps d'afficahge d'une image.
 * \param animation_sequence  La sequence d'animation à moifier.
 */
void MLV_add_frame_in_animation_sequence(
	MLV_Image** array_of_images, MLV_Sound** array_of_sounds,  unsigned int delay,
	MLV_Animation_sequence* animation_sequence
);

#ifdef __cplusplus
}
#endif

#endif

