/*
 *   This file is part of the MLV Library.
 *
 *   Copyright (C) 2010 Adrien Boussicault, Marc Zipstein
 *
 *
 *    This Library is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This Library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this Library.  If not, see <http://www.gnu.org/licenses/>.
 */

/** \~french 
 * \file MLV_window.h
 * \brief Ce fichier définit les prototypes des fonctions permettant de créer,
 * intialiser et fermer des fenêtres.
 *
 * \author Adrien Boussicault
 * \author Marc Zipstein
 *
 */


#ifndef __MLV_WINDOW_H__
#define __MLV_WINDOW_H__

#ifdef __cplusplus
extern "C" {
#endif

/** \~french  
 * \brief Créé une fenêtre dont la taille, et les différents noms sont passés en
 *        paramètres.
 *
 * Cette fonction intialise la librairie MLV.
 * Un programme donné ne peut pas ouvrir plus d'une fenêtre à la
 * fois.
 * Les parametre largeur et hauteur peuvent éventuellement prendre des valeur 
 * négative. Dans ce cas, la fenêtre est dessiné en plein écran.
 *
 * \bug BOGUES WINDOWS UNIQUEMENT\n\n
 *      Sous window uniquement, la fenêtre crée ne peux pas toujours etre ferme 
 *      à l'aide de la croix. En effet, la librairie MLV implemente
 *      une fermeture automatique de la fenetre lorsque l'utilisateur utilise
 *      la croix. Or ce mechanisme utilise les thread SDL qui ne sont 
 *      malheureusement pas bien supporté sous windows.\n\n
 *      En fait, la librairie MLV utilise un second threads pour 
 *      recuperer en permanancer tous les evenement SDL de la pile  et pour les 
 *      traiter au fur et à mesure de leur apparition. OR, sous windwos, comme
 *      les thread ne fonction pas, la librairie ne peux pas lire les evenemnt 
 *      systeme comme ceux de la fermeture de la fenêtre. Elle n'est donc plus 
 *      informée de ce qui se passe.
 *      Pour permettre à la librairie de recupérer ces informations, il vous 
 *      suffit de demander à lire un evenement à l'aide de la fonction :
 *      MLV_get_event()
 *      cela aura pour effet d'obliger la librairie MLV de depiler tout les 
 *      evenements SDL de la pile et de les traiter pour pouvoir repondre à 
 *      votre requête. La librairie MLV en profite alors pour traitere les 
 *      evenements systeme comme la fermeture de la fenetre.
 *      Cela explique aussi pourquoi, lorsque cous utilisez les fonctions :
 *         MLV_wait_keyboard()
 *         MLV_wait_mouse()
 *      la croix se remet à fonction de nouveaux.
 *
 * \param window_name Le nom de la fenêtre
 * \param icone_name Le nom de l'icône de la fenêtre
 * \param width la largeur de la fenêtre
 * \param height la hauteur de la fenêtre
 */
void MLV_create_window(
	const char* window_name, const char* icone_name, 
	unsigned int width, unsigned int height
);

/** \~french  
 * \brief Créé une fenêtre en plein écran dont les différents noms sont passés 
 *        en paramètres.
 *
 * Cette fonction intialise la librairie MLV.
 * Un programme donné ne peut pas ouvrir plus d'une fenêtre à la
 * fois.
 *
 * \param window_name Le nom de la fenêtre
 * \param icone_name Le nom de l'icône de la fenêtre
 * \param width la largeur de la fenêtre ( en mode plein écran, cette valeur 
 *              correspond à la coordonnée en X de la résolution de l'image )
 * \param height la hauteur de la fenêtrei ( en mode plein écran, cela 
 *               correspond à la coordonnée en Y de la résolution de l'image )
 */
void MLV_create_full_screen_window(
	const char* window_name, const char* icone_name,
	unsigned int width, unsigned int height
);

/** \~french  
 * \brief Affiche la fenêtre en plein ecran.
 *
 * La résolution de l'image est donné alors par la taille de l'image.
 * Pour que l'application soit en plein écran, il faut que la taille de la
 * fenetre soit compatible avec la résolution de l'écran.
 */
void MLV_enable_full_screen();

/** \~french  
 * \brief Désactve le mode plein écran de la fenêtre.
 */
void MLV_disable_full_screen();

/** \~french 
 * \brief Test si l'affichage est en mode plein écran ou en mode fenetre.
 *
 * \return Renvoie 1 si l'écran est en mode plein écran, 0 sinon.
 */
int MLV_is_full_screen();

/** \~french  
 * \brief Change la taille de la fenêtre.
 *
 * Si la fenetre a configuré en mode plein ecran, l'appliqcation reste alors 
 * en mode plein écran.
 * Pour que l'application soit en plein écran, il faut que la taille de la 
 * fenetre soit compatible avec la résolution de l'écran.
 * De plus, en mode plein écran, la taille de la fenêtre correspond à la 
 * résolution de l'image sur l'écran.
 *
 * \param width la largeur de la fenêtre
 * \param height la hauteur de la fenêtre
 */
void MLV_change_window_size( unsigned int width, unsigned int height );

/** \~french  
 * \brief Change les intitulés de la fenetre.
 *
 * \param window_name Intitulé apparassaissant sur le fenêtre.
 * \param icone_name  Intitulé apparassaissant dans la barre de la fenêtre.
 */
void MLV_change_window_caption(
	const char* window_name, const char* icone_name
);

/** \~french  
 * \brief Renvoie la taille de la fentre.
 *
 * \param width la largeur de la fenêtre
 * \param height la hauteur de la fenêtre
 */
void MLV_get_window_size( unsigned int* width, unsigned int* height );

/** \~french  
 * \brief Retourne la hauteur de la fenêtre.
 */
int MLV_get_window_height( );

/** \~french  
 * \brief Retourne la largeur de la fenetre.
 */
int MLV_get_window_width( );

/** \~french 
 * \brief Ferme la fenêtre de la librairie MLV.
 *
 * \bug La fonction ecrit le warning suivant :\n\n
 * Error detected by the function unlockPromptInformation() in LibMLV : 
 * Probleme de semaphore ( fichier : prompt.c, ligne : 162)\n\n
 * Il s'agit d'un bogues concernant la libération de la memoire qui est mal 
 * effectué.
 * Les semaphores des prompts sont liberés avant la fin de l'execution des 
 * threads qui l'utilisent. 
 *
 * Cette fonction libère correctement toutes les donées qui ont été crées par la
 * librairie MLV 
 *
 */
void MLV_free_window();

/** \~french  
 * \brief Actualise l'affichage de la fenêtre.
 *
 * Lorsque vous utilisez des fonctions de dessins, les différents changements
 * effectués ne sont pas visibles. Pour les afficher vous devez utiliser cette
 * fonction.
 */
void MLV_actualise_window();

/** \~french  
 * \brief Exécute la fonction de call back donnée en paraètre lorsque le système
 * envoie un signal d'arrêt, ou lorsque l'utilisateur clique sur la croix de la 
 * fenêtre.
 *
 * La fonction de call back est alors exécutée avec en
 * paramêtre un pointeur vers une structure de donnée qui 
 * est aussi donné en paramètre à la fonction 
 * MLV_execute_at_exit.
 *
 * \param function pointeur de la fonction de call back qui  doit etre exécutée.
 * \param data Donnné à passer en parametre à la fonction de call back lors de
 *             son exécution.
 */
void MLV_execute_at_exit( void (*function)(void*), void* data );

#ifdef __cplusplus
}
#endif

#endif
