/*
 *   This file is part of the MLV Library.
 *
 *   Copyright (C) 2010 Adrien Boussicault, Marc Zipstein
 *
 *
 *    This Library is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This Library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this Library.  If not, see <http://www.gnu.org/licenses/>.
 */

/** \~french 
 * \file MLV_xml.h
 *
 * \author Adrien Boussicault
 * \author Marc Zipstein
 *
 * \brief Ce fichier définit les prototypes des fonctions permettant de lire des
 *        fichier xml.
 *
 * Les fichiers xml sont utiliés pour stocker les données nécessaire au 
 * fonctionnnement du programme. Ces fichiers permettent de modifier facilement
 * l'execution du programme sans avoir à modifier le pogramme. En effet, ces
 * sont lisible par l'homme et par la machine en même temps.
 * Ils constituent donc une inteface pratique en entre ordinateur et 
 * utilisateur.
 *
 */

#ifndef __MLV_XML_H__
#define __MLV_XML_H__

#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

/** \~french 
 * \brief Définit le type de donnée stockée au format xml dans la librairie MLV.
 */
typedef struct _MLV_Xml MLV_Xml;

/** \~french 
 * \brief Charge en mémoire un fichier au format xml.
 *
 * Cette fonction renvoi NULL, lorsque le fichier n'a pas pu être correctement
 * chargé.
 *
 * \param xml_file_path Chemin d'accès du fichier xml.
 * \return Le fichier xml lu, anamlysé et chargé.
 */
MLV_Xml* MLV_load_xml( const char* xml_file_path );

/** \~french 
 * \brief Libère la mémoire utilisée par les donnees du fichier xml.
 *
 * \param xml_data les données xml a supprimer.
 */
void MLV_free_xml( MLV_Xml* xml_data );

/** \~french 
 * \brief Permet d'accedr à l'interieur d'un champs donne du fichier xml.
 *
 * \param xml_data les données xml.
 * \param i le ieme champs apparaissant dans le fichier et vérifiant le xpath
 * \param xpath Le chemin d'accesles parametres de la fonction.
 */
char* MLV_get_string_value_from_xml(
	MLV_Xml* xml_data, int i, const char* xpath, ...
);

/** \~french 
 * \brief Permet d'accedr à l'interieur d'un champs donne du fichier xml.
 *
 * \param xml_data les données xml.
 * \param i le ieme champs apparaissant dans le fichier et vérifiant le xpath
 * \param xpath Le chemin d'accesles parametres de la fonction.
 * \param pile La pile des complements de parametre comme dans vaprintf.
 */
char* MLV_get_string_value_from_xml_va(
	MLV_Xml* xml_data, int i, const char* xpath, va_list pile
);


/** \~french 
 * \brief Permet de recupere l'entier d'un champs donne du fichier xml.
 *
 * \param xml_data les données xml.
 * \param i le ieme champs apparaissant dans le fichier et vérifiant le xpath
 * \param xpath Le chemin d'accesles parametres de la fonction.
 */
int MLV_get_integer_value_from_xml(
	MLV_Xml* xml_data, int i, const char* xpath, ...
);

/** \~french 
 * \brief Permet de recupere le reel d'un champs donne du fichier xml.
 *
 * \param xml_data les données xml.
 * \param i le ieme champs apparaissant dans le fichier et vérifiant le xpath
 * \param xpath Le chemin d'accesles parametres de la fonction.
 * \return le reel extrait du xml
 */
double MLV_get_double_value_from_xml(
	MLV_Xml* xml_data, int i, const char* xpath, ...
);

/** \~french 
 * \brief Permet de recupere le réel d'un champs donne du fichier xml.
 *
 * \param xml_data les données xml.
 * \param i le ieme champs apparaissant dans le fichier et vérifiant le xpath
 * \param xpath Le chemin d'accesles parametres de la fonction.
 * \return le reel extrait du xml
 */
float MLV_get_float_value_from_xml(
	MLV_Xml* xml_data, int i, const char* xpath, ...
);


/** \~french 
 * \brief Compte le nombre d'objet présent dans le fichier xml verifiant
 *        un certain nombre de critère passé en paramètre de la fonction.
 *
 * \param xml_data les données xml.
 * \param xpath Le chemin d'accesles parametres de la fonction.
 * \param ... les parametres de la fonction.
 */
int MLV_get_number_of_objects_from_xml( MLV_Xml* xml_data, const char* xpath, ... );

/** \~french 
 * \brief Permet d'acced à l'interieur d'un champs donne du fichier xml.
 *
 * \param xml_data les données xml.
 * \param ... les parametres de la fonction.
 */
MLV_Xml* MLV_get_xml_object_of( MLV_Xml* xml_data, ... );

/** \~french 
 * \brief écrit sur le terminal le contenu d'un fichier xml.
 *
 * \param xml_data les données xml.
 * \param ... les parametres de la fonction.
 */
void MLV_print_xml( MLV_Xml* xml_data );

#ifdef __cplusplus
}
#endif

#endif
