/*
 *   This file is part of the MLV Library.
 *
 *   Copyright (C) 2010 Adrien Boussicault, Marc Zipstein
 *
 *
 *    This Library is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This Library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this Library.  If not, see <http://www.gnu.org/licenses/>.
 */

/** \~french 
 * \file MLV_path.h
 *
 * \author Adrien Boussicault
 * \author Marc Zipstein
 *
 * \brief Ce fichier définit les prototypes des fonctions permettant de
 *        manipuler les chemin d'acces aux fichiers et répertoires.
 *
 */

#ifndef __MLV_PATH_H__
#define __MLV_PATH_H__

#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

/** \~french
 * \brief Renvoie la chaîne de caractères après le dernier séparteur / dans un
 *        chemin donné en paramètre.
 *
 * \param path Le chemin à analyser.
 * \return Une chaîne de caractère.
 */
char* MLV_get_base_name( const char* path );

/** \~french
 *
 * \brief Renvoie la chaîne de caractère s'étendnant jusqu'au dernier séparateur
 *        / du chemin donné en paramètre.
 *
 * \param path Le chemin à analyser.
 * \return Une chaîne de caractère.
 */
char* MLV_get_directory_name( const char* path );

/** \~french
 * \brief Teste si un chemin est absolu.
 *
 * \param path Le chemin
 * \return 1 si le chemin est absolue, 0 sinon.
 */
int MLV_path_is_absolute( const char* path );

/** \~french
 * \brief Teste si un chemin est relatif.
 *
 * \param path Le chemin
 */
int MLV_path_is_relative( const char* path );

/** \~french
 *
 * \brief Teste si le chemin corespond au chemin d'un fichier ou dossier
 *        existant.
 *
 * \param path Le chemin
 * \return Renvoie 1 si le chemin est associé à un dossier ou un fichier
 *         existant, 0 sinon.
 */
int MLV_path_exists( const char* path );

/** \~french
 * \brief Vérifie si un chemin est un associé à un répertoire.
 *
 * \param path Le chemin
 * \return Renvoie 1 si le chemin est associé à un dossier, 0 sinon.
 */
int MLV_path_is_a_directory( const char* path );

/** \~french
 * \brief Vérifie si un chemin est un associé à un fichier.
 *
 * \param path Le chemin
 * \return Renvoie 1 si le chemin est associé à un fichier, 0 sinon.
 */
int MLV_path_is_a_file( const char* path );

/** \~french
 * \brief Construit en concaténant différents textes, une chaine de caractère 
 *        représentant un chemin dont les séparateurs corespondent aux 
 *        spécifications de la plateforme sur laquel s'execute le programme.
 *
 * \bug Cette fonction ne fait que concatener les differentes chaines en
 *      en mettant entre chaque chaine un separateur de repertoire compatible
 *      avec le systeme d'exploitation. Par contre, ce qui constitue le bug, il
 *      ne convertit pas chache chaine dans un format compatible avec celui du
 *      système d'exploitation.
 *
 * \param first_element La premiere chaîne à concatener
 * \param ... les autres éléments à concatener
 * \return Le chemin vérifiant les normes de la plateforme hôte.
 */
char* MLV_build_path( const char* first_element, ... );

/** \~french
 * \brief Même chose que MLV_build_path
 *
 * \param elements les éléments à concatener ensemble
 * \return Le chemin vérifiant les normes de la plateforme hôte.
 */
char* MLV_build_path_v( char** elements );

/** \~french
 * \brief Determine le répertoire courant.
 *
 * \return Le chemin du répertire courant.
 */
const char * MLV_get_current_directory( );

/** \~french
 * \brief Determine le répertoire temporaire.
 *
 * Le répertoire temporaire est le repertoire dans lequel les applications ont
 * ont le droit d'ajouter et de modifier des fichiers.
 * Ce répertoire génaralement vidé lorqsue la machine est étteinte.
 *
 * \return Le chemin du répertire temporaire.
 */
const char * MLV_get_temporary_directory( );

/** \~french
 * \brief Determine le répertoire personnel de l'utilisateur courant.
 *
 * \return Le chemin du répertire personnel.
 */
const char * MLV_get_home_directory( );

#ifdef __cplusplus
}
#endif

#endif
